/*
 * DynamicCore.cc
 * - Core allocator
 */
#include "DynamicCore.hh"

/*
 * Constructor
 */
DynamicCore::DynamicCore(VRInstanceMonitor*** vrInstanceMonitor_par, int* vrInstanceMonitorCount_par) {	// get LVRM monitors dispatched
	vrInstanceMonitor = vrInstanceMonitor_par;
	vrInstanceMonitorCount = vrInstanceMonitorCount_par;
}

/*
 * Get the time-stamp counter
 */
__u64 DynamicCore::getTimeStamp() {	//save hardware TimeStamp Counter to Destination Register
	__u32 hi, lo;
	asm volatile("rdtsc" : "=d" (hi), "=a" (lo));
	return (__u64) hi << 32 | lo;
}

/*
 * Re-allocate the processor cores
 */
void DynamicCore::allocate() {
	int CPU[] = {2, 6, 1, 3, 5, 7};	// CPUs dedicated to VR instances

	if ((getTimeStamp() - timeStamp) / CPU_HZ < 1) {
		return;
	}

	timeStamp = getTimeStamp();

	for (int i = 0; i < *vrInstanceMonitorCount; i++) {
#ifdef PRINT_COUNT
		printf("%i %d\n", i, (*vrInstanceMonitor)[i]->getVRInstanceAdapterCount());
#endif

		if ((*vrInstanceMonitor)[i]->throughput() < VRI_RATE_MAX * ((*vrInstanceMonitor)[i]->getVRInstanceAdapterCount() - 1.5)) {
			for (int j = 0; j < CPU_NO; j++) {	// if low arrival rate, deallocate a VR instance
				if (cpuToVR[j] == (*vrInstanceMonitor)[i]) {
					cpuToVR[j] = 0;
					(*vrInstanceMonitor)[i]->destroyVRInstanceAdapter(CPU[j]);
					break;
				}
			}
		} else if (VRI_RATE_MAX * (*vrInstanceMonitor)[i]->getVRInstanceAdapterCount() <= (*vrInstanceMonitor)[i]->throughput()) {
			for (int j = 0; j < CPU_NO; j++) {	// if high arrival rate, allocate a VR instance
				if (!cpuToVR[j]) {
					cpuToVR[j] = (*vrInstanceMonitor)[i];
					(*vrInstanceMonitor)[i]->createVRInstanceAdapter(CPU[j]);
					break;
				}
			}
		}
	}
}
