/*
 * BusyRingBuffer.hh
 * - Ringbuffer based on busy waiting
 */

#ifndef __BusyRingBuffer_hh__
#define __BusyRingBuffer_hh__

#include <pthread.h>
#include <stdio.h>
#include "common.hh"

#define barrier() __asm__ __volatile__("mfence": : :"memory")

template <class T> class RingBuffer {
	volatile int readIndex;
	volatile int writeIndex;
	volatile int max;				// capacity of the buffer
	volatile int blockOnEmpty;
	struct {
		short size;
		T data;
	} __attribute((packed)) buffer[RB_SIZE];

public:
   
	RingBuffer(int size, bool block=true, int wSyncMinCount=1) {
		fprintf(stderr, "BusyRingBuffer.hh is used\n");
		if (size < 2) size = 2; // shouldn't allow value less than 2.
		readIndex = 0; 
		writeIndex = 0;
		max = size;
		blockOnEmpty = block;
	}

	inline int nextVal(int x) { return (x+1) >= max ? 0 : x+1; }
	// inline int nextVal(int x) { return (x+1)%max; }

	int Insert(T data, int size) {
		int nextWriteIndex = nextVal(writeIndex);
		while (nextWriteIndex == readIndex) {
		}
		buffer[writeIndex].size = size;
		buffer[writeIndex].data = data;
		writeIndex = nextWriteIndex;
		return 0;
	}

	int Extract(T *data) {
		while (readIndex == writeIndex) {
			if (!blockOnEmpty) {
				return -1;
			}
		}
		int size = buffer[readIndex].size;
		*data = buffer[readIndex].data;
		readIndex = nextVal(readIndex);
		return size;
	}

	void PrintStats(char* text) {
	}	

	int dataCount() {
		return readIndex <= writeIndex ? writeIndex - readIndex : max + writeIndex - readIndex;
	}
};

#endif 
