/*
 * LVRM.hh
 */
#ifndef __LVRM_hh__
#define __LVRM_hh__

#ifdef PCapFile	// define and include the socket adapter based on the makefile marco SOCKET_ADAPTER
	#define SOCKET_ADAPTER PCapFile
	#include "SocketAdapter/PCapFile.hh"
#elif defined(PF_RING)
	#define SOCKET_ADAPTER PFRing
	#include "SocketAdapter/PFRing.hh"
#else
	#define SOCKET_ADAPTER Raw
	#include "SocketAdapter/Raw.hh"
#endif

#ifdef DYNAMIC_CORE	// define and include the core allocator based on the makefile marco CORE_ALLOCATOR
	#define CORE_ALLOCATOR DynamicCore
	#include "CoreAllocator/DynamicCore.hh"
#else
	#define CORE_ALLOCATOR FixCore
	#include "CoreAllocator/FixCore.hh"
#endif

#include <linux/ip.h>
#include "VRInstanceMonitor.hh"

extern class LVRM {
public:
	// type definition
	struct KeyToVRIM {							// map MAC address to VR Instance monitor
		char key[IPVERSION + 1];				// ID of the frame, defined by source IP network
		VRInstanceMonitor* vrInstanceMonitor;   // ID of the VR
	} __attribute((packed));

	LVRM();
	VRInstanceMonitor* createVRInstanceMonitor(KeyToVRIM*, int, char*);
	int initialize();
	int transceive();

private:
	// constant
	const static int CPU = 0;	// core dedicated to the LVRM

	// data members
	CoreAllocator* coreAllocator;			// allocate processor cores to VRs
	KeyToVRIM *keyToVRIM;					// map keys to VR instance monitors
	SocketAdapter* socketAdapter;			// connect to socket
	VRInstanceMonitor** vrInstanceMonitor;	// VR instance monitor
	int keyCount;							// count the number of keys to VR instance monitors
	int vrInstanceMonitorCount;				// count the number of VR instance monitors

	int fromVRInstanceMonitor(VRInstanceAdapter::Buffer*);
	int toVRInstanceMonitor(VRInstanceAdapter::Buffer*, int);
	static void handler(int);
	static void waiter(int);
	void createKeyToVRIM(KeyToVRIM*);
	void destroyKeyToVRIM(KeyToVRIM*);
	void destroyVRInstanceMonitor(VRInstanceMonitor*);
	~LVRM();
}* lvrm;	// "main.cc" defines lvrm

#endif
