/*
 * WeightedArrivalTime.cc
 * - estimate the throughput by the arrival rate
 */
#include "WeightedArrivalTime.hh"
#include <stdio.h>

/*
 * Constructor
 */
WeightedArrivalTime::WeightedArrivalTime() : weightedEstimate(1 / 0.) {	// initialize the time-stamp
}

/*
 * Calculate the estimate
 */
float WeightedArrivalTime::estimate() {	// calculate the the difference from the current arrival to the previous arrival
	if (timeStamp) {
		__u64 timeStamp_ = timeStamp;
		float currentEstimate = (timeStamp = getTimeStamp()) - timeStamp_;

		if (weightedEstimate < 1 / 0.) {
			return weightedEstimate = (currentEstimate + weight * weightedEstimate) / (1 + weight);
		}

		return weightedEstimate = currentEstimate;
	}

	return timeStamp = getTimeStamp();
}

/*
 * Get the previous estimate without re-calculate
 */
float WeightedArrivalTime::getEstimate() {	// access the arrival time
	return weightedEstimate;
}

/*
 * Get the time-stamp counter
 */
__u64 WeightedArrivalTime::getTimeStamp() {	//save hardware TimeStamp Counter to Destination Register
	__u32 hi, lo;
	asm volatile("rdtsc" : "=d" (hi), "=a" (lo));
	return (__u64) hi << 32 | lo;
}
