/*
 * PCapFile.cc
 * - connect to pcap file offline
 */
#include "PCapFile.hh"
#include <sys/mman.h>

/*
 * Constructor
 */
PCapFile::PCapFile() {
	char errbuf[PCAP_ERRBUF_SIZE], filename[] = "/home/fwchoi/in.pcap", *pHead = pCurrent = (char*) malloc(6200009080), *pkt;
	pcap_t *pcap = pcap_open_offline(filename, errbuf);
	struct pcap_pkthdr hdr;
	int i = 0;

	for (; (pkt = (char*) pcap_next(pcap, &hdr)) && i < PKT_NO; i++) {
		*(short*) pCurrent = hdr.caplen;
		memcpy(pCurrent += sizeof(short), pkt, hdr.caplen);
		pCurrent += hdr.caplen;
	}

	printf("%d\n", i);
	*(short*) pCurrent = timeStamp[1] = 0;
	pCurrent = pHead;
	timeStamp[0] = getTimeStamp();
}

/*
 * Get the time-stamp counter
 */
__u64 PCapFile::getTimeStamp() {	//save hardware TimeStamp Counter to Destination Register
	__u32 hi, lo;
	asm volatile("rdtsc" : "=d" (hi), "=a" (lo));
	return (__u64) hi << 32 | lo;
}

/*
 * Drop the network frame from the VLRM
 */
int PCapFile::fromLVRM(char*, int) {	// return no byte sent
	return 0;
}

/*
 * Try to receive a network frame from the socket to the LVRM
 */
int PCapFile::toLVRM(char* buffer) {
	int len = *(short*) pCurrent;

	if (!len) {
		printf("%f\n", (getTimeStamp() - timeStamp[0]) / CPU_HZ);
		kill(0, SIGINT);
	}

	memcpy(buffer, pCurrent += sizeof(short), len);
	pCurrent += len;
	return len;
}

/*
 * Try to receive a network frame from the socket to the LVRM
 */
void PCapFile::end() {
	if (1 < timeStamp[1]) {
		printf("%f\n", (getTimeStamp() - timeStamp[1]) / CPU_HZ);
		timeStamp[1] = 1;
	}
}

/*
 * Try to receive a network frame from the socket to the LVRM
 */
void PCapFile::start() {
	if (!timeStamp[1]) {
		timeStamp[0] = timeStamp[1] = getTimeStamp();
	}
}
