/*
 * PFRing.cc
 * - connect to PF_RING
 */
#include "PFRing.hh"
#include <fcntl.h>

/*
 * Constructor
 */
PFRing::PFRing() {
	char device_name[][9] = {"eth0.101", "eth1", {0}};	// symbolic name of the PF_RING-aware device we are attempting to open

	for (deviceNameCounter = 0; device_name[deviceNameCounter][0];) {	// count all the device names
		deviceNameCounter++;
	}

	fd = (int*) malloc(deviceNameCounter * sizeof(int));	// allocate memory for the socket descriptors and the PF_RING handles
	ringtable = (pfring**) malloc(deviceNameCounter * sizeof(pfring*));

	for (int i = 0; i < deviceNameCounter; i++) {	// initialize all sockets and PF_RINGs
		sockaddr_ll sa = {AF_PACKET, htons(ETH_P_ALL)};
		fd[i] = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
		sa.sll_ifindex = nf_nametoindex(device_name[i]);
		bind(fd[i], (sockaddr *) &sa, sizeof sa);
		ringtable[i] = pfring_open(device_name[i], 1, ETH_FRAME_LEN, 0);
		pfring_set_direction(ringtable[i], rx_only_direction);
		pfring_enable_ring(ringtable[i]);
		if_nametomac(device_name[i], (__u8*) mac[i]);
	}
}

/*
 * Send the network frame from the VLRM to the raw socket
 */
int PFRing::fromLVRM(char* buffer, int buffer_len) {	// return the number of bytes sent to the socket w.r.t. the source MAC address
	for (int i = 0; i < deviceNameCounter; i++) {
		if (!memcmp(buffer + ETH_ALEN, mac[i], ETH_ALEN)) {
			return send(fd[i], buffer, buffer_len, 0);
		}
	}

	return 0;	// return no byte sent
}

/*
 * try to receive a network frame from the PF_RING to the LVRM
 */
int PFRing::toLVRM(char* buffer) {
	struct pfring_pkthdr header;	// a memory area where the packet header will be copied

	for (int i = 0; i < deviceNameCounter; i++) {	// if an incoming packet is ready, return the number of bytes received
		if (0 < pfring_recv(ringtable[i], buffer, ETH_FRAME_LEN, &header, 0) && header.len < ETH_FRAME_LEN) {
			return header.len;
		}
	}

	return 0;	// return no byte received
}
