/*
 * Raw.cc
 * - connect to raw socket
 */
#include "Raw.hh"

/*
 * Constructor
 */
Raw::Raw() {
	char device_name[][9] = {"eth0.101", "eth1", {0}};	// symbolic name of the PF_RING-aware device we're attempting to open
	sockaddr_ll sa = {AF_PACKET, htons(ETH_P_ALL)};

	for (deviceNameCounter = 0; device_name[deviceNameCounter][0];) {	// count all the device names
		deviceNameCounter++;
	}

	fd = (int*) malloc(deviceNameCounter * sizeof(int));	// allocate memory for file descriptor of the sending and receiving sockets

	for (int i = 0; i < deviceNameCounter; i++) {	// initialize all the sockets
		fd[i] = socket(PF_PACKET, SOCK_RAW, htons(ETH_P_ALL));
		sa.sll_ifindex = nf_nametoindex(device_name[i]);
		bind(fd[i], (sockaddr*) &sa, sizeof sa);
		fcntl(fd[i], F_SETFL, O_NONBLOCK);
		if_nametomac(device_name[i], (__u8*) mac[i]);
	}
}

/*
 * Send the network frame from the VLRM to the socket
 */
int Raw::fromLVRM(char* buffer, int buffer_len) {	// return the number of bytes sent to the socket w.r.t. the source MAC address
	for (int i = 0; i < deviceNameCounter; i++) {
		if (!memcmp(buffer + ETH_ALEN, mac[i], ETH_ALEN)) {
			return send(fd[i], buffer, buffer_len, 0);
		}
	}

	return 0;	// return no byte sent
}

/*
 * Try to receive a network frame from the socket to the LVRM
 */
int Raw::toLVRM(char* buffer) {
	int len;
	sockaddr_ll sa;
	socklen_t addrlen;

	for (int i = 0; i < deviceNameCounter; i++)	// if an incoming IP packet is ready, return the number of bytes received
		if (0 < (len = recvfrom(fd[i], buffer, ETH_FRAME_LEN, 0, (sockaddr *) &sa, &addrlen)) && sa.sll_pkttype != PACKET_OUTGOING)
			return len;

	return 0;	// return no byte received
}
