/*
 * FourRingBuffers.hh
 * - Connect to VLRM with four ring buffers
 */
#ifndef __FourRingBuffers_hh__ // VR instance API
#define __FourRingBuffers_hh__
#include "LVRMAdapter.hh"

class FourRingBuffers : public LVRMAdapter {
	// constants
	const static int SHMMAX = 33554432;			// max seg size

	// data members
	RingBuffer<LVRMAdapter::Buffer>* ringBuffer[4];

	/*
	 * Extract a message from a ring buffer to the VR instance
	 */
	int fromLVRM(LVRMAdapter::Buffer* buffer) {	// try to receive a control message from the control ring buffer
		int len = ringBuffer[CTRL_TO_VR_INSTANCE]->Extract(buffer) - sizeof buffer->sd /*- sizeof buffer->timeStamp*/;

		if (0 < len) {	// if a control message is ready, return the number of bytes received from the control ring buffer
			return len;
		}

		return ringBuffer[DATA_TO_VR_INSTANCE]->Extract(buffer) - sizeof buffer->sd /*- sizeof buffer->timeStamp*/;
	}	// return the number of bytes received from the data ring buffer

	/*
	 * Insert the message from the VR instance via a ring buffer to an instance
	 */
	int toLVRM(char* buffer_par, int len, __u8 sd) {
		LVRMAdapter::Buffer buffer;	// initialize the network frame with the receiving instance ID and data
		buffer.sd = sd;	// initialize the network frame with the receiving instance ID and data
		memcpy(buffer.buffer, buffer_par, len);

		if (sd)	// if it is to another VR instance, return the number of bytes sent to the control ring buffer
#if defined(BASIC_RING_BUFFER) || defined(BUSY_RING_BUFFER) || defined(CP_BATCH_RING_BUFFER_3)
			return ringBuffer[CTRL_TO_LVRM]->Insert(buffer, len + sizeof buffer.sd /*+ sizeof buffer.timeStamp*/);

		return ringBuffer[DATA_TO_LVRM]->Insert(buffer, len + sizeof buffer.sd /*+ sizeof buffer.timeStamp*/);
#endif
			return ringBuffer[CTRL_TO_LVRM]->Insert(&buffer, len + sizeof buffer.sd /*+ sizeof buffer.timeStamp*/);

		return ringBuffer[DATA_TO_LVRM]->Insert(&buffer, len + sizeof buffer.sd /*+ sizeof buffer.timeStamp*/);
	}	// return the number of bytes sent to data ring buffer

public:
	/*
	 * Constructor
	 */
	FourRingBuffers(int shmid_par) {
		int buffer_len[] = {SHMMAX / 4,	// ring buffer 0
			SHMMAX / 4,					// ring buffer 1
			SHMMAX / 4,					// ring buffer 2
			SHMMAX / 4};				// ring buffer 3

		ringBuffer[0] = (RingBuffer<LVRMAdapter::Buffer>*) (shmaddr = (char*) shmat(shmid = shmid_par, (void*) SHM_ADDR, SHM_RND));

		if (shmaddr == (void*) -1 && errno == EINVAL) {
			ringBuffer[0] = (RingBuffer<LVRMAdapter::Buffer>*) SHM_ADDR;
			shmaddr = (char*) SHM_ADDR;
		}

		for (int i = 1; i < 4; i++) {	// calculate other pointers to ring buffers
			ringBuffer[i] = (RingBuffer<LVRMAdapter::Buffer>*) ((char*) ringBuffer[i - 1] + buffer_len[i - 1]);
		}
	}
};

#endif
