/*
 * LVRMAdapter.hh
 * - Connect to VLRM
 */
#ifndef __LVRMAdapter_hh__ // VR instance API
#define __LVRMAdapter_hh__

#include <linux/if_ether.h>
#include <sys/shm.h>

#define BUSY_RING_BUFFER

#ifdef BASIC_RING_BUFFER
	#include "../RingBuffer/BasicRingBuffer.hh"
#elif defined(BUSY_RING_BUFFER)
	#include "../RingBuffer/BusyRingBuffer.hh"
#elif defined(CP_BATCH_RING_BUFFER_3)
	#include "../RingBuffer/CPBatchRingBuffer3.hh"
#else 
	#include "../RingBuffer/LockRingBuffer.hh"
#endif

#define SHM_ADDR 0x700000000000	// address tried to be attached

class LVRMAdapter {
protected:
	// const
	const static int DATA_TO_VR_INSTANCE = 0;	// index of the data ring buffer from the LVRM to the VR instance
	const static int DATA_TO_LVRM = 1;			// index of the data ring buffer from the VR instance to the LVRM
	const static int CTRL_TO_VR_INSTANCE = 2;	// index of the control ring buffer from the LVRM to the VR instance
	const static int CTRL_TO_LVRM = 3;			// index of the control ring buffer from the VR instance to the LVRM

	// data members
	char* shmaddr;	// address attached
	int shmid;		// shared memory segment ID

	/*
	 * Destructor
	 */
	~LVRMAdapter() {
		shmdt(shmaddr);					// DeTach SHared Memory segment located at address specified by RING buf from address space
		shmctl(shmid, IPC_RMID, NULL);	// mark segment to be destroyed
	}

public:
	// type definition
	struct Buffer {
		__u8 sd;					// source/destination instance
//		__u64 timeStamp;			// timestamp
		char buffer[ETH_FRAME_LEN];	// ethernet frame buffer
	} __attribute((packed));

	virtual int fromLVRM(Buffer *buf) = 0;
	virtual int toLVRM(char*, int, __u8) = 0;
};

#endif
