#include <click/config.h>
#include <click/confparse.hh>
#include <click/standard/scheduleinfo.hh>
#include <unistd.h>
#include "fromLVRM.hh"
CLICK_DECLS

__u64 rdtsc() {
	__u32 hi, lo;
	asm volatile("rdtsc" : "=d" (hi), "=a" (lo));
	return (__u64) hi << 32 | lo;
}

FromLVRM::FromLVRM() : task(this) {
	n = t = ts = 0;
}

int FromLVRM::configure(Vector<String>& conf, ErrorHandler* errh) {
	if (cp_va_parse(conf, this, errh, cpUnsigned, "SHMID", &shmid, cpEnd) < 0) {
		return -1;
	}

	return 0;
}

int FromLVRM::initialize(ErrorHandler* errh) {
	lvrmAdapter = new LVRM_ADAPTER(shmid);
	ScheduleInfo::initialize_task(this, &task, true, errh);
	return 0;
}

bool FromLVRM::run_task(Task*) {
	if (ts) {
		n++;
	}

	LVRMAdapter::Buffer buf;					// receiving buffer
	int len = lvrmAdapter->fromLVRM(&buf);	//length of network frame received

	if (0 < len) {
		ts = rdtsc();
		Packet* p = Packet::make(0, buf.buffer, len, 0);
		output(0).push(p);
		fprintf(stderr, "\r");
	} else {
		ts = ts = 0;
	}

	task.fast_reschedule();
	return 0;
}

CLICK_ENDDECLS
EXPORT_ELEMENT(FromLVRM)
ELEMENT_MT_SAFE(FromLVRM)
