/*
 * MultiQueue.cc
 * - Connect to VLRM with multi-queues
 */

#include "MultiQueue.hh"

/*
 * Constructor
 */
MultiQueue::MultiQueue(int shmid_par) {
	int buffer_len[] = {SHMMAX / 4,	// IPC buffer 0
		SHMMAX / 4,					// IPC buffer 1
		SHMMAX / 4,					// IPC buffer 2
		SHMMAX / 4};				// IPC buffer 3

	ipcBuffer[0] = (RingBuffer<LVRMAdapter::Buffer>*) (shmaddr = (char*) shmat(shmid = shmid_par, (void*) SHM_ADDR, SHM_RND));	// IPC buffer 0

	for (int i = 1; i < 4; i++) {	// calculate other pointers to IPC buffers
		ipcBuffer[i] = (RingBuffer<LVRMAdapter::Buffer>*) ((char*) ipcBuffer[i - 1] + buffer_len[i - 1]);
	}
}

/*
 * Extract a message from a IPC buffer to the VR instance
 */
int MultiQueue::fromLVRM(LVRMAdapter::Buffer* buffer) {	// try to receive a control message from the control ring buffer
	int len = ipcBuffer[CTRL_TO_VR_INSTANCE]->Extract(buffer) - sizeof buffer->sd;	// actual frame size

	if (0 < len) {	// if a control message is ready, return the number of bytes received from the control ring buffer
		return len;
	}

	return ipcBuffer[DATA_TO_VR_INSTANCE]->Extract(buffer) - sizeof buffer->sd;
}	// return the number of bytes received from the data ring buffer

/*
 * Insert the message from the VR instance via a ring buffer to an instance
 */
int MultiQueue::toLVRM(char* buffer_par, int len, __u8 sd) {
	LVRMAdapter::Buffer buffer;	// temporary buffer for the frame from the VR instance to the LVRM
	buffer.sd = sd;				// initialize the network frame with the receiving instance ID and data
	memcpy(buffer.buffer, buffer_par, len);

	if (sd) {	// if it is to another VR instance, return the number of bytes sent to the control ring buffer
		return ipcBuffer[CTRL_TO_LVRM]->Insert(buffer, len + sizeof buffer.sd);
	}

	return ipcBuffer[DATA_TO_LVRM]->Insert(buffer, len + sizeof buffer.sd);
}	// return the number of bytes sent to data ring buffer
