#include <arpa/inet.h>
#include <linux/ip.h>
#include <stdlib.h>
#include "MultiQueue.hh"

int main(int, char** argv) {
	LVRMAdapter::Buffer buffer;	// initialize some variables for the main loop
	const ethhdr* eth = (ethhdr*) ((char*) &buffer + sizeof buffer.sd);
	const iphdr* ip = (iphdr*) ((char*) eth + ETH_HLEN);
	LVRMAdapter* lvrmAdapter = new MultiQueue(atoi(argv[1] + 6));

	char path[strlen(argv[0]) + strlen(".route") + sizeof('\0')];
	sprintf(path, "%s.route", argv[0]);
	FILE* stream = fopen(path, "r");
	int keyCount, len, rtn = fscanf(stream, "%d", &keyCount);

	struct {								// forwarding information base
		__be32 daddr;						// destination IP network
		int mask;							// destination IP network mask
		unsigned char h_dest[ETH_ALEN];		// destination MAC address
		unsigned char h_source[ETH_ALEN];	// source MAC address
	} __attribute((packed)) routeTable[keyCount];

	for (int i = 0; i < keyCount; i++) {	// read the IP networks after IP network count read
		int tmp;

		for (int j = 0; j < (int) sizeof(__be32) - 1; j++) {	// read IP network's octets 1-by-1
			rtn = fscanf(stream, "%d.", &tmp);
			((char*) &routeTable[i].daddr)[j] = tmp;
		}

		rtn = fscanf(stream, "%d/", &tmp);	// read IP network's last octet
		((char*) &routeTable[i].daddr)[sizeof(__be32) - 1] = tmp;
		rtn = fscanf(stream, "%d", &tmp);	// read IP network mask
		routeTable[i].mask = tmp;

		for (int j = 0; j < ETH_ALEN - 1; j++) {	// read destination MAC's octets 1-by-1
			rtn = fscanf(stream, "%x:", &tmp);
			routeTable[i].h_dest[j] = tmp;
		}

		rtn = fscanf(stream, "%x", &tmp);	// read destination MAC's last octet
		routeTable[i].h_dest[ETH_ALEN - 1] = tmp;

		for (int j = 0; j < ETH_ALEN - 1; j++) {	// read source MAC's octets 1-by-1
			rtn = fscanf(stream, "%x:", &tmp);
			routeTable[i].h_source[j] = tmp;
		}

		rtn = fscanf(stream, "%x", &tmp);	// read source MAC's last octet
		routeTable[i].h_source[ETH_ALEN - 1] = tmp;
	}

	while (1) {	// main loop routes IP packets with repective to destination MAC address
		if (0 < (len = lvrmAdapter->fromLVRM(&buffer)) && (eth->h_proto == 8)) {
			for (int i = 0; routeTable[i].daddr; i++) {
				if (ntohl(routeTable[i].daddr) == (ntohl(ip->daddr) & -1 << (32 - routeTable[i].mask))) {
					memcpy((void*) eth, routeTable[i].h_dest, 2 * ETH_ALEN);
					lvrmAdapter->toLVRM((char*) eth, len, 0);
					break;
				}
			}
		}
	}
}
