/*
 * VRInstanceAdapter.hh
 */
#ifndef __VRInstanceAdapter_hh__
#define __VRInstanceAdapter_hh__

#ifdef QUEUE_LENGTH	// define and include the latency estimator based on the makefile marco LATENCY_ESTIMATOR
	#define LATENCY_ESTIMATOR QueueLength
	#include "../LoadEstimator/QueueLength.hh"
#else
	#define LATENCY_ESTIMATOR WeightedQueueLength
	#include "../LoadEstimator/WeightedQueueLength.hh"
#endif

#include <linux/if_ether.h>
#include <signal.h>
#include <stdlib.h>
#include <sys/shm.h>
#include <sys/wait.h>
#include <unistd.h>
#include "../IPCBuffer/BusyRingBuffer.hh"	// include the IPC buffers]

class VRInstanceAdapter {
protected:
	// constants
	const static int DATA_TO_LVRM = 1;			// index of the data ring buffer from the VR instance to the LVRM
	const static int CTRL_TO_VR_INSTANCE = 2;	// index of the control ring buffer from the LVRM to the VR instance
	const static int CTRL_TO_LVRM = 3;			// index of the control ring buffer from the VR instance to the LVRM

	// data members
	char *shmaddr;	// attaching address
	int cpu;		// ID of the CPU dedicated
	int shmid;		// shared memory segment ID
	pid_t pid;		// process ID of the child VR instance process

public:
	// constants
	const static int DATA_TO_VR_INSTANCE = 0;	// index of the data ring buffer from the LVRM to the VR instance
	const static int SHMMAX = 33554432;			// max seg size

	// type definition of a RingBuffer slot
	struct Buffer {
		__u8 sd;					// source/destination instance number
		char buffer[ETH_FRAME_LEN];	// ethernet frame buffer
	} __attribute((packed));

	// data members
	RingBuffer<Buffer> **ipcBuffer;
	LoadEstimator* latencyEstimator;

	~VRInstanceAdapter();
	int fromVRInstance(Buffer*);
	int getCpu();
	int toVRInstance(Buffer*, int);
	void initialize(char*, int);
};

#endif
