/*
 * VRInstanceMonitor.hh
 */
#ifndef __VRInstanceMonitor_hh__
#define __VRInstanceMonitor_hh__

#ifdef ARRIVAL_TIME	// define and include the load estimator based on the makefile marco THROUGHPUT_ESTIMATOR
	#define THROUGHPUT_ESTIMATOR ArrivalTime
	#include "LoadEstimator/ArrivalTime.hh"
#else
	#define THROUGHPUT_ESTIMATOR WeightedArrivalTime
	#include "LoadEstimator/WeightedArrivalTime.hh"
#endif

#ifdef JSQ	// define and include the load balancer based on the makefile marco LOAD_BALANCER
	#define LOAD_BALANCER jsq
	#include "LoadBalancer/jsq.hh"
#elif defined(RND)
	#define LOAD_BALANCER rnd
	#include "LoadBalancer/rnd.hh"
#elif defined(RR)
	#define LOAD_BALANCER rr
	#include "LoadBalancer/rr.hh"
#else
	#define LOAD_BALANCER wrr
	#include "LoadBalancer/wrr.hh"
#endif

#ifdef TWO_RING_BUFFERS	// define and include the VR instance adapter based on the makefile marco VR_INSTANCE_ADAPTER
	#define VR_INSTANCE_ADAPTER TwoRingBuffers
	#include "VRInstanceAdapter/TwoRingBuffers.hh"
#else
	#define VR_INSTANCE_ADAPTER VRInstanceAdapter
	#include "VRInstanceAdapter/VRInstanceAdapter.hh"
#endif

class VRInstanceMonitor {
	// constant
	const static float CPU_HZ = 2.4e9;	// /proc/cpuinfo defines CPU frequency

	// data members
	LoadBalancer* loadBalancer;				// balance load within the VR
	LoadEstimator* throughputEstimator;		// estimate the total throughput of the VR
	VRInstanceAdapter** vrInstanceAdapter;	// connect to VR instance
	char* vrName;							// name of the VR program
	int vrInstanceAdapterCount;				// count the number of VR instance adapter

public:
	VRInstanceMonitor(char*);
	float throughput();
	int fromVRInstanceAdapter(VRInstanceAdapter::Buffer*);
	int getVRInstanceAdapterCount();
	int toVRInstanceAdapter(VRInstanceAdapter::Buffer*, int);
	void createVRInstanceAdapter(int);
	void destroyVRInstanceAdapter(int);
	~VRInstanceMonitor();
};

#endif
